# Clone or update the yolov7 repo
import os, subprocess, shutil
os.chdir('tabledetect')

if os.path.exists('yolov7'):
    # Sync
    pass
else:   
    # Clone
    command = 'git clone https://github.com/WongKinYiu/yolov7.git yolov7'; subprocess.call(command, shell=True)

# Add train_custom.py.backup if applicable


import os
from PIL import Image
import json

def yoloBboxToPillowBbox(yoloBbox, image_width, image_height):
    # Convert YOLO coordinates to pixel coordinates
    x_center, y_center, width, height = yoloBbox['x_center'], yoloBbox['y_center'], yoloBbox['width'], yoloBbox['height']
    left = int((x_center - width / 2) * image_width)
    top = int((y_center - height / 2) * image_height)
    right = int(left + width * image_width)
    bottom = int(top + height * image_height)
    
    return (left, top, right, bottom)
    
def extractCroppedImages(bbox_lists_per_file_list, outDir, imageFormat, imageDir, saveBoundingBoxFile):
    # Create folders
    uniqueLabelIds = set(bbox['labelId'] for item in bbox_lists_per_file_list for _, bbox_list in item.items() for bbox in bbox_list)
    os.makedirs(os.path.join(outDir), exist_ok=True)
    [os.makedirs(os.path.join(outDir, str(labelId)), exist_ok=True) for labelId in uniqueLabelIds]

    # Loop over source images
    boundingBoxInfos = []
    for bbox_lists_per_file in bbox_lists_per_file_list: 
        for filename, bbox_list in bbox_lists_per_file.items():
            imageName = f'{filename}{imageFormat}'
            imagePath = os.path.join(imageDir, imageName)
            inputImage = Image.open(imagePath)

            # Loop over object in source image
            for i, bbox in enumerate(bbox_list):
                pathOut = os.path.join(outDir, str(bbox['labelId']), f'{filename}_{i}{imageFormat}')
                
                pillowBbox = yoloBboxToPillowBbox(yoloBbox=bbox, image_width=inputImage.width, image_height=inputImage.height)
                cropped = inputImage.crop(pillowBbox)
                cropped.save(pathOut)

                if saveBoundingBoxFile:
                    boundingBoxInfos.append({'sourceName': filename, 'labelId': bbox['labelId'], 'table_num': i, 'pillowBbox': pillowBbox})

    if saveBoundingBoxFile:
        pathBoundingBoxFile = os.path.join(outDir, 'boundingBoxData.json')
        with open(pathBoundingBoxFile, 'w') as boundingBoxFile:
            json.dump(boundingBoxInfos, boundingBoxFile)


import os
def getBoundingBoxesPerFile(yoloAnnotationsFilePath):
    with open(yoloAnnotationsFilePath) as yoloAnnotationsFile:
        filename = os.path.splitext(os.path.basename(yoloAnnotationsFilePath))[0]
        bbox_lists_per_file = {filename: []}
        for line in yoloAnnotationsFile:
            yoloAnnotation = line.strip()
            labelId, x_center, y_center, width, height, conf = map(float, yoloAnnotation.split(' '))
            bbox_lists_per_file[filename].append({
                                'labelId': int(labelId),
                                'x_center': x_center, 'y_center': y_center,
                                'width': width, 'height': height,
                                'confidence': conf})
    return bbox_lists_per_file


# Imports
import os, subprocess, sys, shutil
from pathlib import Path
from tabledetect.helpers.yolo_to_boundingbox import getBoundingBoxesPerFile
from tabledetect.helpers.boundigbox_to_cropped_image import extractCroppedImages
from tabledetect.helpers.download import downloadRepo, downloadWeights
import logging; logging.basicConfig(level=logging.INFO)

# Check if torch and torchvision installed
try:
    import torch
    import torchvision
except ModuleNotFoundError:
    raise ModuleNotFoundError('pytorch module not found, go to https://pytorch.org/get-started/locally/ to install the correct version')

# Constants
PATH_PACKAGE = os.path.dirname(os.path.realpath(__file__))
PATH_WEIGHTS = os.path.join(PATH_PACKAGE, 'resources', 'tabledetect.pt')
PATH_WEIGHTS_URL = 'https://www.dropbox.com/s/k1iuhwk2k62uifb/tabledetect.pt?dl=1'
PATH_EXAMPLES = os.path.join(PATH_PACKAGE, 'resources', 'examples')
PATH_PYTHON = sys.executable
PATH_OUT = Path(PATH_PACKAGE).parent / 'out'

PATH_ML_MODEL = os.path.join(PATH_PACKAGE, 'yolov7-main')
PATH_SCRIPT_DETECT = os.path.join(PATH_PACKAGE, 'yolov7-main', 'detect_codamo.py')
if not os.path.exists(PATH_SCRIPT_DETECT):
    downloadRepo(url='https://github.com/Danferno/yolov7/archive/master.zip', destination=PATH_PACKAGE)

if not os.path.exists(PATH_WEIGHTS):
    downloadWeights(url=PATH_WEIGHTS_URL, destination=PATH_WEIGHTS)


# Detect tables
def detect_table(path_input=PATH_EXAMPLES, path_cropped_output=os.path.join(PATH_OUT, 'cropped'), device=None, threshold_confidence=0.5, model_image_size=992, trace='--no-trace', image_format='.png', save_bounding_box_file=True, verbosity=logging.INFO):
    # Parse options
    logging.basicConfig(level=verbosity)
    logging.debug('Checking if torch is properly installed')
    if not device:
        device = torch.device("cuda:0" if torch.cuda.is_available() else "cpu")
    if not image_format.startswith('.'):
        image_format = f'.{image_format}'

    # Detect
    logging.info('Detecting objects in your source files')
    if os.path.exists(PATH_OUT):
        shutil.rmtree(PATH_OUT)
    command = f'{PATH_PYTHON} "{PATH_SCRIPT_DETECT}"' \
                f' --weights {PATH_WEIGHTS}' \
                f' --conf {threshold_confidence}' \
                f' --img-size {model_image_size}' \
                f' --source {path_input}' \
                f' --save-txt --save-conf' \
                f' --project out --name table-detect' \
                f' {trace}'
    subprocess.run(command, check=True)

    # Extract bounding boxes
    logging.info('Extracting bounding box information from the YOLO files')
    bbox_lists_per_file = [getBoundingBoxesPerFile(annotationfile.path) for annotationfile in os.scandir(os.path.join(PATH_OUT, 'table-detect', 'labels'))]

    # Crop images
    logging.info('Extracting cropped images and saving single bounding box json file')
    extractCroppedImages(bbox_lists_per_file_list=bbox_lists_per_file, outDir=path_cropped_output, imageFormat=image_format, imageDir=path_input, saveBoundingBoxFile=save_bounding_box_file)



    


import logging
import requests
import zipfile as zf
from io import BytesIO
def downloadRepo(url, destination):
    logging.info(f'ML model repository not found.\nDownloading from: {url}\nDestination: {destination}')
    response = requests.get(url)

    with zf.ZipFile(BytesIO(response.content)) as zipfile:
        zipfile.extractall(destination)
    logging.info(f'Finished downloading ML model repository.')

def downloadWeights(url, destination):
    logging.info(f'ML weights not found.\nDownloading from: {url}\nDestination: {destination}')
    response = requests.get(url)

    with open(destination, 'wb') as out:
        out.write(response.content)
    logging.info(f'Finished downloading ML weights.')